/**
 * Service for account
 * @extends TheService
 * @class TheAccountService
 */
'use strict'

const path = require('path')
const { isBase64, saveBase64 } = require('the-server-util')
const { TheService } = require('the-service-base')

class TheAccountService extends TheService {
  /**
   * Process updating user password
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processPassword (config) {
    const {
      newPassword,
      userId,
    } = config
    const { Sign, User } = this.resources
    const user = await User.one(userId)
    const sign = await Sign.setUserPassword(user, newPassword)
    return { sign, user }
  }

  /**
   * Process updating user profile
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processProfile (config) {
    const {
      onImage,
      profileAttributes: { image },
      profileAttributes,
      publicDir = 'public',
      userId,
    } = config
    const { Profile, User } = this.resources
    const user = await User.one(userId)
    const profile = await Profile.ofUser(user)

    if (image) {
      if (isBase64(image)) {
        profileAttributes.image = await this._saveProfileImage({
          data: [].concat(image)[0],
          publicDir,
          user,
        })
        onImage && onImage(profileAttributes.image)
      }
    }
    await profile.update(profileAttributes)
    await user.update({ profile })
    return { profile, user }
  }

  async _saveProfileImage ({ data, publicDir, user }) {
    const dirname = path.resolve(publicDir, 'account/images')
    const { filename } = await saveBase64(dirname, String(user.id), data)
    return '/' + path.relative(publicDir, filename)
  }
}

module.exports = TheAccountService
