/**
 * Service for sign
 * @extends TheService
 * @class TheSignService
 */
'use strict'

const { TheError, TheNotFoundError } = require('the-error')
const { TheService } = require('the-service-base')
const WrongPasswordError = TheError.withName('WrongPasswordError')

/** @lends TheSignService */
class TheSignService extends TheService {
  /**
   * Process sign-in
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processSignIn (config) {
    const { lang, name, password } = config
    const { Profile, Sign, User } = this.resources
    const user = (await User.first({ name })) || (await Profile.userWithEmail(name))
    if (!user) {
      throw new TheNotFoundError(`User not found for name: ${name}`, {
        field: 'name',
        messageKey: 'USER_NOT_FOUND_ERROR',
      })
    }
    const sign = await Sign.ofUser(user)
    const ok = sign.verifyPassword(password)
    if (!ok) {
      throw new WrongPasswordError('Password wrong', {
        field: 'password',
        messageKey: 'PASSWORD_WRONG',
      })
    }
    const profile = await Profile.ofUser(user)
    await sign.update({ signInAt: new Date() })
    await user.update({ lang: lang || user.lang, profile, sign })
    return { sign, user }
  }

  /**
   * Process signout
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processSignOut (config) {
    const { userId } = config
    const { Sign, User } = this.resources
    const user = await User.one(userId)
    const sign = await Sign.ofUser(user)
    await Sign.update(sign.id, { signOutAt: new Date() })
    await user.update({ sign })
    return { sign }
  }

  /**
   * Process sign-up
   * @param {Object} config
   * @returns {Promise<object>}
   */
  async processSignUp (config) {
    const {
      lang = null,
      name,
      password,
      profileAttributes = {},
      roleCode = null,
    } = config

    const { Profile, Role, Sign, User } = this.resources
    const role = roleCode && await Role.ofCode(roleCode)
    const user = await User.create({
      lang,
      name: name && String(name).trim().toLowerCase(),
      role,
    })
    try {
      const profile = await Profile.ofUser(user)
      await profile.update(
        Object.assign({}, profileAttributes, { emailVerified: false }),
        { errorNamespace: 'profile' }
      )
      await user.update({ profile })
    } catch (e) {
      await user.destroy()
      throw e
    }
    const sign = await Sign.setUserPassword(user, password)
    await user.update({ sign })
    return { sign, user }
  }
}

module.exports = TheSignService
