/**
 * Service to manage user
 * @extends TheService
 * @class TheUserService
 */
'use strict'

const { TheService } = require('the-service-base')

const toId = ({ id }) => id

class TheUserService extends TheService {
  /**
   * Pick list of users
   * @param {Object} config
   * @returns {Promise<void|*>}
   */
  async pickList (config) {
    const { filter, page, sort } = config
    const { User } = this.resources
    return User.list({ filter, page, sort })
  }

  /**
   * Process creating user
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processCreate (config) {
    const {
      name,
      profileAttributes,
      roleCode = null,
    } = config
    const { Profile, Role, Sign, User } = this.resources
    const role = roleCode && await Role.ofCode(roleCode)
    const user = await User.create({ name, role })
    user.password = await Sign.resetPasswordForUser(user)
    try {
      const profile = await Profile.create(
        Object.assign(
          {},
          profileAttributes,
          { user }
        ), { errorNamespace: 'profile' })
      await user.update({ profile })
      return { profile, role, user }
    } catch (e) {
      await user.destroy()
      throw e
    }
  }

  /**
   * Process destroying users
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processDestroy (config) {
    const {
      userIds,
    } = config
    const { Profile, Sign, User } = this.resources
    const users = await Promise.all(userIds.map((userId) => User.one(userId)))
    const signs = await Sign.all({ user: users })
    const profiles = await Profile.all({ user: users })
    return {
      destroyed: {
        profiles: await Profile.destroyBulk(profiles.map(toId)),
        signs: await Sign.destroyBulk(signs.map(toId)),
        users: await User.destroyBulk(users.map(toId)),
      },
    }
  }

  /**
   * Process reset user password
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processReset (config) {
    const {
      userIds,
    } = config
    const { Sign, User } = this.resources
    const newPasswords = {}
    for (const id of userIds) {
      const user = await User.one(id)
      if (!user) {
        continue
      }
      newPasswords[id] = await Sign.resetPasswordForUser(user)
    }
    return { newPasswords }
  }

  /**
   * Process update user role
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processRole (config) {
    const {
      roleCode,
      userIds,
    } = config
    const { Role, User } = this.resources
    const role = await Role.ofCode(roleCode)
    const newRoles = {}
    for (const id of userIds) {
      const user = await User.one(id)
      if (!user) {
        continue
      }
      await user.update({ role })
      newRoles[id] = user.role.code
    }
    return { newRoles }
  }
}

module.exports = TheUserService
