/**
 * Service for verify
 * @extends TheService
 * @class TheVerifyService
 */
'use strict'

const theDate = require('the-date')
const { TheError, TheExpiredError, TheInvalidParameterError } = require('the-error')
const { TheService } = require('the-service-base')
const VerifySendError = TheError.withName('VerifySendError')

const { Durations: { ONE_HOUR } } = theDate

class TheVerifyService extends TheService {
  /**
   * Pick verify needed
   * @param {Object} config
   * @returns {Promise<boolean>}
   */
  async pickNeeded (config) {
    const {
      userId,
    } = config
    const { User } = this.resources
    const user = await User.one(userId)
    const { profile } = user
    return Boolean(profile && profile.isEmailVerifyNeeded)
  }

  /**
   * Process prepare for verify
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processPrepare (config) {
    const {
      expireIn = 48 * ONE_HOUR,
      userId,
    } = config
    const { User } = this.resources
    const user = await User.one(userId)
    const { email } = user.profile || {}
    if (!email) {
      throw new VerifySendError(`Email is not registered`)
    }

    const expireAt = Number(theDate().add(expireIn).toDate())
    const envelop = { email, expireAt, userId: user.id }

    return { envelop, expireAt }
  }

  /**
   * Process to verify
   * @param {Object} config
   * @returns {Promise<Object>}
   */
  async processVerify (config) {
    const {
      envelop,
    } = config
    const { Profile, Sign, User } = this.resources

    const { email, expireAt, userId } = envelop
    const isExpired = new Date(Number(expireAt)) < new Date()
    if (isExpired) {
      throw new TheExpiredError('Verify expired')
    }
    await User.assertUserNotGone(userId)
    const user = await User.one(userId)
    const sign = await Sign.ofUser(user)
    const profile = await Profile.ofUser(user)
    if (profile.email !== email) {
      throw new TheInvalidParameterError(`Invalid parameter`, envelop)
    }
    await profile.update({ emailVerified: true })
    return { profile, sign, user }
  }
}

module.exports = TheVerifyService
